// AccountsAndPositionsDlg.h : header file
//

#pragma once

class CAccountsAndPositionsDlg;

/**
\typedef ATL::IDispEventImpl<1, CAccountsAndPositionsDlg, 
                              &__uuidof(_ICQGCELEvents), 
                              &__uuidof(__CQG), 4, 0> ICQGCELDispEventImpl
\brief  Convenient definition of IDispEventImpl instantiation
*/
typedef ATL::IDispEventImpl<1,
                              CAccountsAndPositionsDlg,
                              &__uuidof(_ICQGCELEvents),
                              &__uuidof(__CQG), 4, 0
                           >
                           ICQGCELDispEventImpl;

/// \brief CAccountsAndPositionsDlg dialog. 
/// \brief Used to show accounts and positions data.
class CAccountsAndPositionsDlg : public CDialog, public ICQGCELDispEventImpl
{

public:

   CAccountsAndPositionsDlg(CWnd* pParent = NULL);

   /// \enum Dialog Data
   enum { IDD = IDD_ACCOUNTSANDPOSITIONS_DIALOG };

   /// \brief Prepares the container for painting.
   afx_msg void OnPaint();

   /// \brief Called by a minimized (iconic) window that does not have an icon defined for its class.
   afx_msg HCURSOR OnQueryDragIcon();

   /// \brief Handles accounts combo box selected account changed event.
   /// \brief Used to update all data for currently selected account.
   afx_msg void OnCbnSelChangeCmbAccounts();

   /// \brief Handles report days combo box selected item changed event.
   /// \brief Used to update summaries for currently selected report day.
   afx_msg void OnCbnSelchangeCmbReportDays();

   /// \brief Handles account currencies combo box selected item changed event.
   /// \brief Used to update summaries for currently selected currency.
   afx_msg void OnCbnSelchangeCmbCurrencies();

   /// \brief Used to unadvise from CQGCEL events on dialog destroy.
   afx_msg void OnDestroy();

   /// \brief Handles context menu popup event
   afx_msg void OnContextMenu(CWnd* /*pWnd*/, CPoint /*point*/);

   /// \brief Handles particular context menu item's click
   afx_msg void OnPopupNomargin();
   afx_msg void OnPopupMarginsnapshot();
   afx_msg void OnPopupMarginsnapshotandupdates();

   /// \brief Declares that the class defines a message map.
   DECLARE_MESSAGE_MAP()

protected:

   /// \brief Dialog icon.
   HICON m_hIcon;

   /// \brief Exchanges and validates dialog data.
   virtual void DoDataExchange(
                  CDataExchange* pDX);	// DDX/DDV support

   /// \brief This message is sent to the dialog box immediately before the dialog box is displayed.
   /// \return Specifies whether the application has set the input 
   /// \return focus to one of the controls in the dialog box
   virtual BOOL OnInitDialog();

   /// \brief Override to prevent dialog closing on Enter.
   virtual void OnOK();

public:

   /// \brief This map is used to declare handler function for specified event.
   /// \brief You can use "OLE/COM Object Viewer" to open CQGCEL-4_0.dll and get event ids.
   /// \brief The handler signature as well as the event ids can be found in the type library
   /// \brief or shown by "OLE/COM Object Viewer".
   BEGIN_SINK_MAP(CAccountsAndPositionsDlg)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 10, OnDataError)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 2, OnGWConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 3, OnDataConnectionStatusChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 7, OnAccountChanged)
      SINK_ENTRY_EX(1, __uuidof(_ICQGCELEvents), 102, OnInstrumentResolved)
   END_SINK_MAP()

protected:

   /// \brief Handle event of some abnormal discrepancy
   /// \brief between data expected and data received from CQG.
   /// \brief This event also can be fired when CQGCEL startup fails.
   /// \param obj - Object where the error occurred
   /// \param errorDescription - String describing the error
   /// \return S_OK
   STDMETHOD(OnDataError)(
                  LPDISPATCH obj,
                  BSTR errorDescription);

   /// \brief Fired when some changes occurred
   /// \brief in the connection with the CQG Gateway.
   /// \param newStatus - New connection status
   /// \return S_OK
   STDMETHOD(OnGWConnectionStatusChanged)(
                  eConnectionStatus newStatus);

   /// \brief Fired when some changes occurred
   /// \brief in the connection with the CQG data server.
   /// \param newStatus - New connection status
   /// \return S_OK
   STDMETHOD(OnDataConnectionStatusChanged)(
                  eConnectionStatus newStatus);

   /// \brief Fired when new account is added, changed or removed.
   /// \param change - Account change type
   /// \param account - Changed account instance
   /// \param position - Changed position instance
   /// \return S_OK
   STDMETHOD(OnAccountChanged)(
                  eAccountChangeType change,
                  ICQGAccount* account,
                  ICQGPosition* position);

   /// \brief Fired when a new instrument is resolved and subscribed or the requested symbol is not available.
   /// \param symbol - Requested symbol
   /// \param cqg_instrument - Subscribed instrument object
   /// \param cqg_error - Error occured during a symbol resolution process
   /// \return S_OK on success
   STDMETHOD(OnInstrumentResolved)(BSTR symbol, ICQGInstrument* cqg_instrument, ICQGError* cqg_error);

private:

   /// \brief Initialize all controls.
   void InitializeControls();

   /// \brief Initialize account properties list view.
   void InitializeAccProperties();

   /// \brief Initialize account summaries list view.
   void InitializeAccSummary();

   /// \brief Initialize positions list view.
   void InitializePositions();

   /// \brief Initialize reporting days combo for account summaries.
   void InitializeReportDays();

   /// \brief Initialize and starts CQGCEL object.
   void InitializeCQGCEL();

   /// \brief Shows account summaries by values selected in report days
   /// \brief and currencies combos.
   /// \param spAccount - ICQGAccount instance
   void ShowSummaries(
                  const ATL::CComPtr<ICQGAccount>& spAccount);

   /// \brief Shows account summaries calculated for specified report day.
   /// \brief One summary is by reporting currency and another one by specified currency.
   /// \param spAccount - ICQGAccount instance
   /// \param currency - The second summary is calculated by this currency
   /// \param reportDay - Reporting day
   void ShowAccountSummaries(
                  const ATL::CComPtr<ICQGAccount>& spAccount, 
                  ATL::CComBSTR currency, 
                  int reportDay);

   /// \brief Shows account summary calculated for specified report day in the 
   /// \brief specified column in the list control.
   /// \param spAccount - ICQGAccount instance
   /// \param reportDay - Reporting day
   /// \param column - List column where summary is dumped
   void ShowAccountSummary(
                  const ATL::CComPtr<ICQGAccountSummary>& spAccSummary, 
                  int reportDay,
                  int column);

   /// \brief Shows account positions by adding to list control.
   /// \brief Before positions are added old ones are removed. 
   /// \param spAccount - ICQGAccount instance
   void ShowPositions(const ATL::CComPtr<ICQGAccount>& spAccount);

   /// \brief Shows position the list control.
   /// \brief New row is added for position if not found in list.
   /// \param spPosition - ICQGPosition instance
   void ShowPosition(const ATL::CComPtr<ICQGPosition>& spPosition);

   /// \brief Shows account properties in the list control.
   /// \param spAccount - ICQGAccount instance
   void ShowProperties(const ATL::CComPtr<ICQGAccount>& spAccount);

   /// \brief Fills all accounts to combo box control and changes 
   /// \brief their subscription level.
   void ReloadAccounts();

   /// \brief Fills all currencies for specified account to the currencies combo box.
   /// \param spAccount - ICQGAccount instance
   void FillCurrencies(const ATL::CComPtr<ICQGAccount>& spAccount);

   /// \brief Adds account GW name and GWID to the accounts combo box control.
   /// \param spAccount - ICQGAccount instance
   void InsertAccount(const ATL::CComPtr<ICQGAccount>& spAccount);

   /// \brief Shows currently selected account's all data.
   void ShowSelectedAccountData();

   /// \brief Determines if the specified account selected in the accounts list.
   /// \param spAccount - ICQGAccount instance
   /// \return True if specified account is selected
   bool IsAccountSelected(const ATL::CComPtr<ICQGAccount>& spAccount) const;

   /// \brief Determines if the specified account selected in the accounts list.
   /// \return ICQGAccount instance
   ATL::CComPtr<ICQGAccount> GetSelectedAccount() const;

   /// \brief Clears all data from the form shown for account
   void ClearAccountData();

   /// \brief Used for getting CQGCEL properties values string representation.
   /// \brief If specified value is invalid(see CQGCEL.IsValid() method) then "N/A" is returned.
   /// \param value - Variant value
   /// \return String representation of the value
   CString GetValueAsString(const ATL::CComVariant& value) const;

   /// \brief Enables/disables controls on the form
   /// \param enable The flag to enable or disable the controls
   void EnableControls(bool enable);

   /// \brief Sets the account detailing level of the selected account
   /// \param detailingLevel The detailing level to be set
   void SetCurrentAccountLevel(eAccountMarginDetailing detailingLevel);

   /// \brief Returns string representation of the connection status.    
   /// \param status - connection status
   /// \return String representation of the connection status
   static CString ConnectionStatusToString(eConnectionStatus status);

   /// \brief Return the string representation of the account margin detailing
   /// \param detailingValue The detailing level value to convert
   /// \return The string representation
   static CString AccountMarginDetailingToString(eAccountMarginDetailing detailingValue);

private:

   /// \enum eAccPropertiesRow. Represents account properties rows in the list
   enum eAccPropertiesRow;

   /// \enum eAccPropertiesColumn. Represents account properties columns in the list
   enum eAccPropertiesColumn;

   /// \enum eAccSummaryRow. Represents account summaries rows in the list
   enum eAccSummaryRow;

   /// \enum eAccSummaryColumn. Represents account summaries columns in the list
   enum eAccSummaryColumn;

   /// \enum ePositionColumn. Represents account positions columns in the list
   enum ePositionColumn;

private:

   /// \brief CQGCEL object
   ATL::CComPtr<ICQGCEL> m_spCQGCEL;

   /// \brief Data connection indicator label control
   CStatic m_lblDataConnection;

   /// \brief GW connection indicator label control
   CStatic m_lblGWConnection;

   /// \brief Accounts combo box control
   /// \brief Contains account's GW names, as well as the GWID as item data.
   /// \brief GWID is used to get account instance from CQGCEL accounts collection.
   CComboBox m_cmbAccounts;

   /// \brief Summary reporting days combo box control.
   /// \brief Account summaries can be calculated by date. This control contains
   /// \brief the reporting days.
   CComboBox m_cmbReportDays;

   /// \brief Account currencies combo box control.
   /// \brief Contains all available currencies for account.
   CComboBox m_cmbCurrencies;

   /// \brief Account summaries combo box control.
   /// \brief Each account has one reporting summary calculated with reporting currency.
   /// \brief Also, account can have summaries calculated by other currencies. 
   /// \brief These summaries(currencies) depend on account. 
   /// \brief This list shows one summary for reporting currency and one for selected currency from 
   /// \brief currencies combo box.
   CListCtrl m_lvAccSummary;

   /// \brief Account positions list control.
   CListCtrl m_lvPositions;
   
   /// \brief Account properties list control.
   CListCtrl m_lvAccProperties;

   /// \brief Account summaries list column width.
   static const int m_stSummaryColWidth; 

   /// \brief Not available value
   static const CString m_N_A;
};

enum CAccountsAndPositionsDlg::eAccPropertiesRow
{
   aprFCMID,
   aprFCMAccountID,
   aprFCMName,
   aprGWAccountName,
   aprGWAccountID,
   aprDateOfLastStatement,
   aprReportingCurrency,
   aprTimestampOfLastUpdate,
   aprTotalMargin,
   aprTotalPurchasingPower,
   aprMarginDetailingLevel,
   aprPropertiesRowsCount // Account properties rows count
};

enum CAccountsAndPositionsDlg::eAccPropertiesColumn
{
   apcName,
   apcValue,
};

enum CAccountsAndPositionsDlg::eAccSummaryRow
{
   asrBalance,
   asrPL,
   asrOTE,
   asrUPL,
   asrCollaterals,
   asrNLV,
   asrMVO,
   asrCashExcess,
   asrInitialMargin,
   asrMaintenanceMargin,
   asrSummaryRowsCount // Account summary rows count
};

enum CAccountsAndPositionsDlg::eAccSummaryColumn
{
   ascName,
   ascRepCur,
   ascCur,
};

enum CAccountsAndPositionsDlg::ePositionColumn
{
   pcInstrument,
   pcLong,
   pcShort,
   pcAveragePrice,
   pcOTE,
   pcMVO,
   pcOffset,
   pcPL,
   pcCurrency
};
